@@ -21,6 +21,7 @@
 
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
+const Signals = imports.signals;
 const Gettext = imports.gettext;
 const _ = imports.gettext.gettext;
 
@@ -44,6 +45,7 @@ const Manager = imports.manager;
 const Miners = imports.miners;
 const Notifications = imports.notifications;
 const Path = imports.path;
+const Properties = imports.properties;
 const Query = imports.query;
 const Selections = imports.selections;
 const Sources = imports.sources;
@@ -59,6 +61,8 @@ const Application = new Lang.Class({
     Extends: Gtk.Application,
 
     _init: function() {
+        this.minersRunning = [];
+
         Gettext.bindtextdomain('gnome-documents', Path.LOCALE_DIR);
         Gettext.textdomain('gnome-documents');
         GLib.set_prgname('gnome-documents');
@@ -117,16 +121,28 @@ const Application = new Lang.Class({
     },
 
     _onActionPrintCurrent: function() {
-        let doc = Global.documentManager.getActiveItem();;
+        let doc = Global.documentManager.getActiveItem();
         if (doc)
             doc.print(this._mainWindow.window);
     },
 
-    _onActionSearch: function(action) {
+    _onActionToggle: function(action) {
         let state = action.get_state();
         action.change_state(GLib.Variant.new('b', !state.get_boolean()));
     },
 
+    _onActionProperties: function() {
+        let doc = Global.documentManager.getActiveItem();
+        if (!doc)
+            return;
+
+        let dialog = new Properties.PropertiesDialog(doc.id);
+        dialog.widget.connect('response', Lang.bind(this,
+            function(widget, response) {
+                widget.destroy();
+            }));
+    },
+
     _initActions: function() {
         let actionEntries = [
             { name: 'quit',
@@ -135,12 +151,18 @@ const Application = new Lang.Class({
             { name: 'about',
               callback: this._onActionAbout },
             { name: 'help',
-              callback: this._onActionHelp },
+              callback: this._onActionHelp,
+              accel: 'F1' },
             { name: 'fullscreen',
               callback: this._onActionFullscreen,
               create_hook: this._fullscreenCreateHook,
               accel: 'F11',
               window_mode: WindowMode.WindowMode.PREVIEW },
+            { name: 'gear-menu',
+              callback: this._onActionToggle,
+              state: GLib.Variant.new('b', false),
+              accel: 'F10',
+              window_mode: WindowMode.WindowMode.PREVIEW },
             { name: 'view-as',
               callback: this._onActionViewAs,
               create_hook: this._viewAsCreateHook,
@@ -150,11 +172,11 @@ const Application = new Lang.Class({
             { name: 'open-current',
               callback: this._onActionOpenCurrent,
               window_mode: WindowMode.WindowMode.PREVIEW },
-            { name: 'print-current',
+            { name: 'print-current', accel: '<Primary>p',
               callback: this._onActionPrintCurrent,
               window_mode: WindowMode.WindowMode.PREVIEW },
             { name: 'search',
-              callback: this._onActionSearch,
+              callback: this._onActionToggle,
               state: GLib.Variant.new('b', false),
               accel: '<Primary>f' },
             { name: 'find-next', accel: '<Primary>g',
@@ -174,7 +196,10 @@ const Application = new Lang.Class({
             { name: 'select-all', accel: '<Primary>a',
               window_mode: WindowMode.WindowMode.OVERVIEW },
             { name: 'select-none',
-              window_mode: WindowMode.WindowMode.OVERVIEW }
+              window_mode: WindowMode.WindowMode.OVERVIEW },
+            { name: 'properties',
+              callback: this._onActionProperties,
+              window_mode: WindowMode.WindowMode.PREVIEW }
         ];
 
         actionEntries.forEach(Lang.bind(this,
@@ -224,8 +249,17 @@ const Application = new Lang.Class({
         if (env)
             return false;
 
+        this.minersRunning.push(miner);
+        this.emitJS('miners-changed', this.minersRunning);
+
         miner.RefreshDBRemote(Lang.bind(this,
             function(res, error) {
+                this.minersRunning = this.minersRunning.filter(
+                    function(element) {
+                        return element != miner;
+                    });
+                this.emitJS('miners-changed', this.minersRunning);
+
                 if (error) {
                     log('Error updating the cache: ' + error.toString());
                     return;
@@ -240,6 +274,35 @@ const Application = new Lang.Class({
         return false;
     },
 
+    _refreshMiners: function() {
+        if (Global.sourceManager.hasProviderType('google')) {
+            try {
+                // startup a refresh of the gdocs cache
+                this._refreshMinerNow(this.gdataMiner);
+            } catch (e) {
+                log('Unable to start GData miner: ' + e.message);
+            }
+        }
+
+        if (Global.sourceManager.hasProviderType('windows_live')) {
+            try {
+                // startup a refresh of the skydrive cache
+                this._refreshMinerNow(this.zpjMiner);
+            } catch (e) {
+                log('Unable to start Zpj miner: ' + e.message);
+            }
+        }
+    },
+
+    _initMiners: function() {
+        this.gdataMiner = new Miners.GDataMiner();
+        this.zpjMiner = new Miners.ZpjMiner();
+        this._refreshMiners();
+
+        Global.sourceManager.connect('item-added', Lang.bind(this, this._refreshMiners));
+        Global.sourceManager.connect('item-removed', Lang.bind(this, this._refreshMiners));
+    },
+
     vfunc_startup: function() {
         this.parent();
         String.prototype.format = Format.format;
@@ -277,18 +340,7 @@ const Application = new Lang.Class({
         Global.modeController = new WindowMode.ModeController();
         Global.notificationManager = new Notifications.NotificationManager();
 
-        try {
-          // startup a refresh of the gdocs cache
-          let gdataMiner = new Miners.GDataMiner();
-          this._refreshMinerNow(gdataMiner);
-
-          // startup a refresh of the skydrive cache
-          let zpjMiner = new Miners.ZpjMiner();
-          this._refreshMinerNow(zpjMiner);
-        } catch (e) {
-	  log('Unable to start miners: ' + e.message);
-        }
-
+        this._initMiners();
         this._initActions();
         this._initAppMenu();
         this._mainWindow = new MainWindow.MainWindow(this);
@@ -301,6 +353,8 @@ const Application = new Lang.Class({
     vfunc_command_line: function(commandLine) {
         let args = commandLine.get_arguments();
         if (args.length) {
+            Global.modeController.setWindowMode(WindowMode.WindowMode.PREVIEW);
+
             let urn = args[0]; // gjs eats argv[0]
             let doc = Global.documentManager.getItemById(args[0]);
             if (doc) {
@@ -325,3 +379,4 @@ const Application = new Lang.Class({
         return 0;
     }
 });
+Utils.addJSSignalMethods(Application.prototype);
