@@ -383,9 +383,6 @@ const Embed = new Lang.Class({
         // pack the OSD notification actor
         this._viewActor.add_child(Global.notificationManager.actor);
 
-        // start the monitor for the indexing notification
-        this._indexingNotification = new Notifications.IndexingNotification();
-
         // now create the actual content widgets
         this._view = new View.ViewContainer();
         this._viewPage = this._notebook.append_page(this._view.widget, null);
