@@ -310,7 +310,7 @@ const PreviewFullscreen = new Lang.Class({
         this._filter.stop();
 
         this._thumbBar.actor.destroy();
-        this._fsToolbar.widget.destroy();
+        this._fsToolbar.actor.destroy();
     },
 
     _show: function() {
@@ -373,6 +373,7 @@ const PreviewToolbar = new Lang.Class({
         let previewMenu = this._getPreviewMenu();
         let menuButton = this.widget.add_menu('emblem-system-symbolic', null, false);
         menuButton.set_menu_model(previewMenu);
+        menuButton.set_action_name('app.gear-menu');
 
         this._setToolbarTitle();
         this.widget.show_all();
@@ -401,9 +402,12 @@ const PreviewToolbar = new Lang.Class({
     },
 
     _setToolbarTitle: function() {
-        let doc = Global.documentManager.getActiveItem();
-        let primary = doc.name;
+        let primary = null;
         let detail = null;
+        let doc = Global.documentManager.getActiveItem();
+
+        if (doc)
+            primary = doc.name;
 
         if (this._model) {
             let curPage, totPages;
@@ -446,9 +450,10 @@ const PreviewSearchbar = new Lang.Class({
 
     createSearchWidgets: function() {
         this._searchContainer = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL,
-                                              spacing: 6 });
+                                              spacing: 6,
+                                              halign: Gtk.Align.CENTER});
 
-        this._searchEntry = new Gtk.SearchEntry({ hexpand: true });
+        this._searchEntry = new Gtk.SearchEntry({ width_request: 500 });
         this._searchEntry.connect('activate', Lang.bind(this,
             function() {
                 Global.application.activate_action('find-next', null);
@@ -483,6 +488,7 @@ const PreviewSearchbar = new Lang.Class({
     show: function() {
         this.parent();
 
+        this._searchEntry.select_region(0, -1);
         this._previewView.view.find_set_highlight_search(true);
         this._previewView.startSearch(this._searchEntry.get_text());
     },
