@@ -47,6 +47,8 @@ const QueryFlags = {
 };
 
 const LOCAL_COLLECTIONS_IDENTIFIER = 'gd:collection:local:';
+const TRACKER_SCHEMA = 'org.freedesktop.Tracker.Miner.Files';
+const TRACKER_KEY_RECURSIVE_DIRECTORIES = 'index-recursive-directories';
 
 const Query = new Lang.Class({
     Name: 'Query',
@@ -63,71 +65,91 @@ const QueryBuilder = new Lang.Class({
     _init: function() {
     },
 
+    _getTrackerLocations: function() {
+        let settings = new Gio.Settings({ schema: TRACKER_SCHEMA });
+        let locations = settings.get_strv(TRACKER_KEY_RECURSIVE_DIRECTORIES);
+        let files = [];
+
+        locations.forEach(Lang.bind(this,
+            function(location) {
+                // ignore special XDG placeholders, since we handle those internally
+                if (location[0] == '&' || location[0] == '$')
+                    return;
+
+                let trackerFile = Gio.file_new_for_commandline_arg(location);
+
+                // also ignore XDG locations if they are present with their full path
+                for (let idx = 0; idx < GLib.UserDirectory.N_DIRECTORIES; idx++) {
+                    let file = Gio.file_new_for_path(GLib.get_user_special_dir(idx));
+                    if (trackerFile.equal(file))
+                        return;
+                }
+
+                files.push(trackerFile);
+            }));
+
+        return files;
+    },
+
+    _getBuiltinLocations: function() {
+        let files = [];
+        let xdgDirs = [GLib.UserDirectory.DIRECTORY_DESKTOP,
+                       GLib.UserDirectory.DIRECTORY_DOCUMENTS,
+                       GLib.UserDirectory.DIRECTORY_DOWNLOAD];
+
+        xdgDirs.forEach(Lang.bind(this,
+            function(dir) {
+                let path = GLib.get_user_special_dir(dir);
+                if (path)
+                    files.push(Gio.file_new_for_path(path));
+            }));
+
+        return files;
+    },
+
     buildFilterLocal: function() {
-        let path;
-        let desktopURI;
-        let downloadsURI;
-        let documentsURI;
+        let locations = this._getBuiltinLocations();
+        locations = locations.concat(this._getTrackerLocations());
 
-        path = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_DESKTOP);
-        if (path)
-            desktopURI = Gio.file_new_for_path(path).get_uri();
-        else
-            desktopURI = '';
+        let filters = [];
+        locations.forEach(Lang.bind(this,
+            function(location) {
+                filters.push('(fn:contains (nie:url(?urn), "%s"))'.format(location.get_uri()));
+            }));
 
-        path = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_DOCUMENTS);
-        if (path)
-            documentsURI = Gio.file_new_for_path(path).get_uri();
-        else
-            documentsURI = '';
+        filters.push('(fn:starts-with (nao:identifier(?urn), "gd:collection:local:"))');
 
-        path = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_DOWNLOAD);
-        if (path)
-            downloadsURI = Gio.file_new_for_path(path).get_uri();
-        else
-            downloadsURI = '';
-
-        let filter =
-            ('((fn:contains (nie:url(?urn), "%s")) || ' +
-             ' (fn:contains (nie:url(?urn), "%s")) || ' +
-             ' (fn:contains (nie:url(?urn), "%s")) || ' +
-             ' (fn:starts-with (nao:identifier(?urn), "gd:collection:local:")))').format(desktopURI, documentsURI, downloadsURI);
-
-        return filter;
+        return '(' + filters.join(' || ') + ')';
     },
 
     buildFilterNotLocal: function() {
-        let sparql = '(';
         let sources = Global.sourceManager.getItems();
+        let filters = [];
 
         for (idx in sources) {
             let source = sources[idx];
             if (!source.builtin)
-                sparql += source.getFilter() + ' || ';
+                filters.push(source.getFilter());
         }
 
-        sparql += 'false)';
+        if (filters.length == 0)
+            filters.push('false');
 
-        return sparql;
+        return '(' + filters.join(' || ') + ')';
     },
 
     _buildFilterString: function(currentType) {
-        let sparql = 'FILTER (';
+        let filters = [];
 
-        sparql += Global.searchMatchManager.getFilter();
-        sparql += ' && ';
-        sparql += Global.sourceManager.getFilter();
-        sparql += ' && ';
-        sparql += Global.searchCategoryManager.getFilter();
+        filters.push(Global.searchMatchManager.getFilter());
+        filters.push(Global.sourceManager.getFilter());
+        filters.push(Global.searchCategoryManager.getFilter());
 
         if (currentType) {
-            sparql += ' && ';
-            sparql += currentType.getFilter();
+            filters.push(currentType.getFilter());
         }
 
-        sparql += ')';
-
-        return sparql;
+        return 'FILTER (' + filters.join(' && ') + ')';
     },
 
     _buildOptional: function() {
