@@ -132,6 +132,8 @@ const SearchTypeManager = new Lang.Class({
     Extends: Manager.BaseManager,
 
     _init: function() {
+        // Translators: "Type" refers to a search filter on the document type
+        // (PDF, spreadsheet, ...)
         this.parent(_("Type"));
 
         this.addItem(new SearchType({ id: 'all',
@@ -222,8 +224,10 @@ const SearchMatchManager = new Lang.Class({
         this.addItem(new SearchMatch({ id: SearchMatchStock.ALL,
                                        name: _("All") }));
         this.addItem(new SearchMatch({ id: SearchMatchStock.TITLE,
+        //Translators: "Title" refers to "Match Title" when searching
                                        name: _("Title") }));
         this.addItem(new SearchMatch({ id: SearchMatchStock.AUTHOR,
+        //Translators: "Author" refers to "Match Author" when searching
                                        name: _("Author") }));
 
         this.setActiveItemById(SearchMatchStock.ALL);
@@ -535,15 +539,9 @@ const OverviewSearchbar = new Lang.Class({
     },
 
     createSearchWidgets: function() {
-        this._searchContainer = new Gd.MarginContainer({ min_margin: 64,
-                                                         max_margin: 128 });
-        this._box = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
-        this._searchContainer.add(this._box);
-
         // create the search entry
-        this._searchEntry = new Gd.TaggedEntry({ width_request: 260,
-                                                 no_show_all: true,
-                                                 hexpand: true });
+        this._searchEntry = new Gd.TaggedEntry({ width_request: 500,
+                                                 no_show_all: true });
         this._searchEntry.connect('tag-clicked',
             Lang.bind(this, this._onTagClicked));
         this._searchEntry.set_text(Global.searchController.getString());
@@ -564,9 +562,12 @@ const OverviewSearchbar = new Lang.Class({
                 this._dropdownButton.set_active(false);
             }));
 
-        this._box.add(this._searchEntry);
-        this._box.add(this._dropdownButton);
-        this._box.show_all();
+        this._searchContainer = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL,
+                                              halign: Gtk.Align.CENTER });
+
+        this._searchContainer.add(this._searchEntry);
+        this._searchContainer.add(this._dropdownButton);
+        this._searchContainer.show_all();
     },
 
     entryChanged: function() {
