@@ -57,14 +57,18 @@ const Source = new Lang.Class({
     },
 
     getFilter: function() {
-        if (this.id == SourceStock.LOCAL)
-            return Global.queryBuilder.buildFilterLocal();
+        let filters = [];
 
-        if (this.id == SourceStock.ALL)
-            return '(' + Global.queryBuilder.buildFilterLocal() + ' || '
-                    + Global.queryBuilder.buildFilterNotLocal() + ')';
+        if (this.id == SourceStock.LOCAL) {
+            filters.push(Global.queryBuilder.buildFilterLocal());
+        } else if (this.id == SourceStock.ALL) {
+            filters.push(Global.queryBuilder.buildFilterLocal());
+            filters.push(Global.queryBuilder.buildFilterNotLocal());
+        } else {
+            filters.push(this._buildFilterResource());
+        }
 
-        return this._buildFilterResource();
+        return '(' + filters.join(' || ') + ')';
     },
 
     _buildFilterResource: function() {
@@ -132,5 +136,26 @@ const SourceManager = new Lang.Class({
             });
 
         return hasOnline;
+    },
+
+    hasProviderType: function(providerType) {
+        let found = false;
+        this.forEachItem(Lang.bind(this,
+            function(source) {
+                if (!source.object)
+                    return;
+
+                let account = source.object.get_account();
+                if (!account)
+                    return;
+
+                if (found)
+                    return;
+
+                if (account.provider_type == providerType)
+                    found = true;
+            }));
+
+        return found;
     }
 });
